'''
Created on 2023-04-09

@author: wf
'''
from tests.basetest import Basetest
from profiwiki.patch import Patch
import tempfile

class TestPatch(Basetest):
    """
    test Patching
    """
    
    def setUp(self, debug=False, profile=True):
        Basetest.setUp(self, debug=debug, profile=profile)
        
    def getTestfile(self):
        tmp=tempfile.NamedTemporaryFile(prefix="test",suffix=".php")
        return tmp.name
        
    def getPatch(self):
        # Create a temporary directory and a test PHP file with example variables
        test_file_path=self.getTestfile()
        with open(test_file_path, "w") as f:
            f.write("<?php\n\n")
            f.write("$wgSitename = 'MyWiki';\n")
            f.write("$wgLanguageCode = 'en';\n")
            f.write("$wgEnableEmail = true;\n")

        patch = Patch(test_file_path)
        return patch
    
    def checkPatch(self,patch,expected_content):
        """
        check the patch against the expected content
        """
        with open(patch.file_path) as f:
            contents = f.read()
            self.assertIn(expected_content, contents)
            
    def test_add(self):
        """
        test adding  lines
        """
        lines="""$wgRawHtml = true;
$wgAllowImageTag=true;"""
        patch=self.getPatch()
        self.assertEqual(5,len(patch.lines))
        patch.add_text(lines)
        self.assertEqual(7,len(patch.lines))
        patch.add_text(lines)
        self.assertEqual(7,len(patch.lines))
        
    def test_quirk(self):
        lines="""
        // modified by profiwiki 
// allow raw HTML 
$wgRawHtml = true;
// allow images
$wgAllowImageTag=true;
// InstantCommons allows wiki to use images from https://commons.wikimedia.org
$wgUseInstantCommons = true;
// avoid showing (expected) deprecation warnings
error_reporting(E_ERROR | E_WARNING | E_PARSE | E_NOTICE);
// https://www.mediawiki.org/wiki/Extension:UserFunctions
$wgUFEnabledPersonalDataFunctions = [
    'ip',
    'nickname',
    'realname',
    'useremail',
    'username',
];
// work around last line not copied
        """
        patch=self.getPatch()
        patch.add_text(lines)
        patch.file_path=self.getTestfile()
        patch.save()
        patch2=Patch(patch.file_path)
        print (patch2.lines)
        
    def test_save(self):
        """
        test saving after having added a line
        """
        patch=self.getPatch()
        self.assertEqual(5,len(patch.lines))
        patch.add_text("// an extra line")
        patch.save() 
        patch2=Patch(patch.file_path)
        self.assertEqual(6,len(patch2.lines))

    def test_patch_mediawiki_config_var(self):
        """
        test patching a mediawiki configuration
        """
        # Test patching a configuration variable that exists in the file
        patch=self.getPatch()
        patch.patch_mediawiki_config_var("Sitename", "'NewWikiName'")
        patch.save()
        self.checkPatch(patch,"$wgSitename = 'NewWikiName';")


'''
Created on 2021-08-19

@author: wf
'''
from unittest import TestCase
import time
import getpass

class Basetest(TestCase):
    '''
    base test case
    '''
    
    def setUp(self,debug=False,profile=True):
        '''
        setUp test environment
        '''
        TestCase.setUp(self)
        self.debug=debug
        self.profile=profile
        msg=f"test {self._testMethodName}, debug={self.debug}"
        self.profiler=Profiler(msg,profile=self.profile)
        
    def tearDown(self):
        TestCase.tearDown(self)
        self.profiler.time()    
        
    def inPublicCI(self):
        '''
        are we running in a public Continuous Integration Environment?
        '''
        return getpass.getuser() in [ "travis", "runner" ];

class Profiler:
    '''
    simple profiler
    '''
    def __init__(self,msg,profile=True):
        '''
        construct me with the given msg and profile active flag
        
        Args:
            msg(str): the message to show if profiling is active
            profile(bool): True if messages should be shown
        '''
        self.msg=msg
        self.profile=profile
        self.starttime=time.time()
        if profile:
            print(f"Starting {msg} ...")
    
    def time(self,extraMsg=""):
        '''
        time the action and print if profile is active
        '''
        elapsed=time.time()-self.starttime
        if self.profile:
            print(f"{self.msg}{extraMsg} took {elapsed:5.1f} s")
        return elapsed


'''
Created on 2023-04-01

@author: wf
'''
from mwdocker.docker import DockerContainer
from python_on_whales import DockerException
import tempfile

class ProfiWikiContainer():
    """
    a profiwiki docker container wrapper
    """
    def __init__(self,dc:DockerContainer):
        """
        Args:
            dc(DockerContainer): the to wrap
        """
        self.dc=dc
    
    def log_action(self,action:str):
        """
        log the given action
        
        Args:
            action(str): the d
        """
        if self.dc:
            print(f"{action} {self.dc.kind} {self.dc.name}",flush=True)
        else:
            print(f"{action}",flush=True)
        
    def upload(self,text:str,path:str):
        """
        upload the given text to the given path
        """
        with tempfile.NamedTemporaryFile() as tmp:
            self.log_action(f"uploading {tmp.name} as {path} to ")
            with open(tmp.name,"w") as text_file:
                text_file.write(text)
            self.dc.container.copy_to(tmp.name,path)
        
    def killremove(self,volumes:bool=False):
        """
        kill and remove me
        
        Args:
            volumes(bool): if True remove anonymous volumes associated with the container, default=True (to avoid e.g. passwords to get remembered / stuck
        """
        if self.dc:
            self.log_action("killing and removing")
            self.dc.container.kill()
            self.dc.container.remove(volumes=volumes)

    def start_cron(self):
        """
        Starting periodic command scheduler: cron.
        """  
        self.dc.container.execute(["/usr/sbin/service","cron","start"],tty=True)
        
    def install_plantuml(self):
        """
        install plantuml to this container
        """
        # https://gabrieldemarmiesse.github.io/python-on-whales/docker_objects/containers/
        self.dc.container.execute(["apt-get","update"],tty=True)
        self.dc.container.execute(["apt-get","install","-y","plantuml"],tty=True)
        pass
    
    def install_fontawesome(self):
        """
        install fontawesome to this container
        """
        script="""#!/bin/bash
# install fontawesome
# WF 2023-01-25
cd /var/www
curl https://use.fontawesome.com/releases/v5.15.4/fontawesome-free-5.15.4-web.zip -o fontawesome.zip
unzip -o fontawesome.zip
ln -s -f fontawesome-free-5.15.4-web fontawesome
chown -R www-data.www-data fontawesome
cd fontawesome
ln -s svgs/regular svg
cat << EOS > /etc/apache2/conf-available/font-awesome.conf
Alias /font-awesome /var/www/fontawesome
<Directory /var/www/fontawesome>
  Options Indexes FollowSymLinks MultiViews
  Require all granted
</Directory>
EOS
a2enconf font-awesome
"""
        script_path="/root/install_fontawesome"
        self.upload(script,"/root/install_fontawesome")
        # make executable
        self.dc.container.execute(["chmod","+x",script_path])
        self.dc.container.execute([script_path],tty=True)
        try:
            self.dc.container.execute(["service","apache2","restart"])
        except DockerException as e:
            # we expect a SIGTERM
            if not e.return_code==143:
                raise e
        pass



'''
Created on 2023-04-01

@author: wf
'''
import datetime
import json
import tempfile
import platform
import os
from mwdocker.mwcluster import MediaWikiCluster
from mwdocker.docker import DockerApplication
from profiwiki.docker import ProfiWikiContainer
from wikibot3rd.wikiuser import WikiUser
from mwdocker.config import MwClusterConfig
from profiwiki.patch import Patch

class ProfiWiki():
    """
    ProfiWiki
    """
    
    def __init__(self,prefix:str="pw",smw_version="4.1.1",mw_version="1.39.3",port:int=9079):
        """
        constructor
        """        
        self.os_name=platform.system()
        self.os_uname=os.uname()
        self.os_release=platform.release()
        self.args=None
        self.wikiUser=None
        self.config=MwClusterConfig()
        self.config.smwVersion=smw_version
        self.config.prefix=prefix
        self.config.basePort=port
        self.config.sqlPort=port-1
        self.config.port=port
        self.config.versions=[mw_version]
        self.config.container_base_name="pw"
        self.config.extensionNameList=["Admin Links","Diagrams","Header Tabs","ImageMap","ImageLink","MagicNoCache","Maps9",
                               "Mermaid","MsUpload","Nuke","Page Forms","ParserFunctions","PDFEmbed","Renameuser",
                               "Replace Text","Semantic Result Formats","SyntaxHighlight","Variables","UserFunctions"]
        self.config.logo="https://wiki.bitplan.com/images/wiki/thumb/6/63/Profiwikiicon.png/96px-Profiwikiicon.png"
        self.config.__post_init__()
        self.mwCluster=None
        pass
        
    def system_info(self)->str:
        """
        collect system information
        """
        info=f"""os: {self.os_name}"""
        if "Darwin" in info:
            release,_version,_machine=platform.mac_ver()
            info+=f" MacOS {release}"
        else:
            info+=f"{self.os_release}"
        return info
    
    def work(self,args):
        """
        work as instructed by the arguments
        
        Args:
            args(Namespace): the command line arguments
        """
        self.config.fromArgs(args)
        self.wiki_id=f"{self.config.container_base_name}"
        if args.bash:
            cmd=f"docker exec -it {self.config.container_base_name}-mw /bin/bash"
            print(cmd)
            return
        mwApp=self.getMwApp(withGenerate=args.forceRebuild)
        if args.randompassword:
            self.password=self.config.random_password()
            self.wikiUser=self.createOrModifyWikiUser(mwApp,force_overwrite=args.forceuser)
        if args.wikiuser and not self.wikiUser:
            self.createOrModifyWikiUser(mwApp,force_overwrite=args.forceuser)
        if args.all:
            if not self.wikiUser:
                self.wikiUser=self.createOrModifyWikiUser(mwApp,force_overwrite=args.forceuser)
            self.create(mwApp, args.forceRebuild)
            pmw,_pdb=self.getProfiWikiContainers(mwApp)
            pmw.install_fontawesome()
            pmw.install_plantuml()
            self.patch(pmw)
            self.update(mwApp)
            pmw.start_cron()
        if args.create:
            self.create(mwApp, args.forceRebuild)
        if args.check:
            self.check(mwApp)
        if args.down:
            self.down(mwApp,args.forceRebuild)
        if args.list:
            self.list(mwApp)
        if args.plantuml or args.fontawesome or args.cron or args.patch:
            pmw,_pdb=self.getProfiWikiContainers(mwApp)
            if args.plantuml:
                pmw.install_plantuml()
            if args.fontawesome:
                pmw.install_fontawesome()
            if args.cron:
                pmw.start_cron()
            if args.patch:
                self.patch(pmw)
        if args.update:
            self.update(mwApp)
            
    def createOrModifyWikiUser(self,mwApp,force_overwrite:bool=False)->WikiUser:
        """
        create or modify the WikiUser for this profiwiki
        
        Args:
            mwApp(DockerApplication): the mediawiki docker application
            force_overwrite(bool): if True overwrite the wikuser info
        """
        wikiUsers=WikiUser.getWikiUsers(lenient=True)
        if self.wiki_id in wikiUsers and not force_overwrite:
            wikiUser=wikiUsers[self.wiki_id]          
            if self.config.password != wikiUser.getPassword():
                raise Exception(f"wikiUser for wiki {self.wiki_id} already exists but with different password")
            pass
        else:
            wikiUser=mwApp.createWikiUser(self.wiki_id,store=True)
        return wikiUser
    
    def getMwCluster(self,withGenerate:bool=True)->MediaWikiCluster:
        """
        get a mediawiki Cluster for my configuration
        
        Args:
            withGenerate(bool): if True regenerate the configuration files
            
        Returns:
            MediaWikiCluster: the MediaWiki Cluser
        """
        if self.mwCluster is not None:
            return self.mwCluster
        if self.config.verbose:
            print(f"ProfiWiki {self.config.prefix} using port {self.config.port}")
        mwCluster=MediaWikiCluster(config=self.config)
        # generate
        mwCluster.createApps(withGenerate=withGenerate)
        self.mwCluster=mwCluster
        return mwCluster
    
    def getMwApp(self,withGenerate:bool=True): 
        """
        get my mediawiki Docker application 
        """   
        mwCluster=self.getMwCluster(withGenerate)
        mwApp=mwCluster.apps[self.config.version]
        return mwApp
    
    def getProfiWikiContainers(self,mwApp:DockerApplication):
        """
        get the two containers - for mediawiki and the database
        
        Args:
            mwApp(DockerApplication): the MediaWiki Docker Application
            
        Returns:
            Tuple(ProfiWikiContainer,ProfiWikiContainer): MediaWiki, Database
        """
        mw,db=mwApp.getContainers()
        pmw=ProfiWikiContainer(mw)
        pdb=ProfiWikiContainer(db)
        return pmw,pdb
    
    def patch(self,pwc:ProfiWikiContainer):
        """
        apply profi wiki patches to the given ProfiWikiContainer
        """
        if not pwc.dc:
            raise("no container to apply patch")
        ls_path="/var/www/html/LocalSettings.php"
        timestamp=datetime.datetime.utcnow().strftime('%Y-%m-%d')
        with tempfile.NamedTemporaryFile(mode='w', prefix='LocalSettings_', suffix='.php') as ls_file:
            pwc.log_action(f"patching {ls_file.name}")
            pwc.dc.container.copy_from(ls_path,ls_file.name)
            patch=Patch(file_path=ls_file.name)
            lines=f"""// modified by profiwiki 
# make this an intranet - comment out if you want this to be a public wiki
# The following permissions were set based on your choice in the installer
$wgGroupPermissions['*']['createaccount'] = false;
$wgGroupPermissions['*']['edit'] = false;
$wgGroupPermissions['*']['read'] = false;
// allow raw HTML 
$wgRawHtml = true;
// allow images
$wgAllowImageTag=true;
// InstantCommons allows wiki to use images from https://commons.wikimedia.org
$wgUseInstantCommons = true;
// avoid showing (expected) deprecation warnings
error_reporting(E_ERROR | E_WARNING | E_PARSE | E_NOTICE);
# # add support for special properties
# see https://www.semantic-mediawiki.org/wiki/Help:$smwgPageSpecialProperties
# Modification date
$smwgPageSpecialProperties[] = '_MDAT';
# Creation date
$smwgPageSpecialProperties[] = '_CDAT';
# Is a new page
$smwgPageSpecialProperties[] = '_NEWP';
# Last editor is
$smwgPageSpecialProperties[] = '_LEDT';
# Media type
$smwgPageSpecialProperties[] = '_MEDIA';
# MIME type
$smwgPageSpecialProperties[] = '_MIME';
// https://www.mediawiki.org/wiki/Extension:UserFunctions
$wgUFEnabledPersonalDataFunctions = ['ip','nickname','realname','useremail','username',];
# increase query limit
$smwgQMaxLimit = 20000;
//Default width for the PDF object container.
$wgPdfEmbed['width'] = 800;
//Default height for the PDF object container.
$wgPdfEmbed['height'] = 1090;
//Allow user the usage of the tag 
$wgGroupPermissions['*']['embed_pdf'] = true;
"""
            patch.add_text(lines)
            patch.save()
            pwc.dc.container.copy_to(ls_file.name,ls_path)

    def update(self,mwApp):
        """
        run the update script
        """
        mwApp.execute("/root/update.sh")
        
    def check(self,mwApp):
        """
        check
        """
        mwApp.check()
            
    def create(self,mwApp,forceRebuild:bool=False):
        """
        create a profiwiki mediawiki
        """
        mwApp.start(forceRebuild=forceRebuild)
        
    def down(self,mwApp,forceRebuild:bool=False):
        """
        shut down the profiwiki base mediawiki
        """
        mwApp.down(forceRebuild=forceRebuild)
        
    def list(self,mwApp):
        """
        list the profi wikis
        """
        print (json.dumps(mwApp.config.as_dict(),indent=2))
        pass


'''
Created on 2023-04-01

@author: wf
'''
import profiwiki

class Version(object):
    '''
    Version handling for ProfiWiki
    '''
    name="pyProfiWiki"
    description="""BITPlan's professional Semantic Mediawiki"""
    version=profiwiki.__version__
    date = '2023-04-01'
    updated = '2023-04-08'
    authors='Wolfgang Fahl'
    doc_url="https://wiki.bitplan.com/index.php/ProfiWiki"
    chat_url="https://github.com/BITPlan/ProfiWiki/discussions"
    cm_url="https://github.com/BITPlan/ProfiWiki"
    license=f'''Copyright 2015-2023 contributors. All rights reserved.
  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0
  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.'''
    longDescription=f"""{name} version {version}
{description}
  Created by {authors} on {date} last updated {updated}"""

        


'''
Created on 2023-04-01

@author: wf
'''
from tests.basetest import Basetest
from profiwiki.profiwiki_core import ProfiWiki
import json
import os
from argparse import ArgumentParser
#from mwdocker.webscrape import WebScrape

class TestProfiWiki(Basetest):
    """
    test ProfiWiki
    """
    
    def setUp(self, debug=False, profile=True):
        """
        setUp the test environment
        """
        Basetest.setUp(self, debug=debug, profile=profile)
        # change the port for the testwiki to not spoil a wiki on the default port
        self.pw=ProfiWiki(prefix="pwt1",port=11000)
        self.mwApp=None
        self.argv=["--prefix","pwt1","--basePort","11000","--sqlBasePort","11001"]
        
    def testConfig(self):
        """
        test the config
        """
        config_dict=self.pw.config.as_dict()
        debug=self.debug
        if debug:
            print(json.dumps(config_dict,indent=2))
        self.assertEqual("pwt1",self.pw.config.prefix)
        
    def getProfiWiki(self,argv=[]):
        """
        get a profiwiki for the given command line arguments
        """
        parser = ArgumentParser()
        pw=ProfiWiki()
        pw.config.addArgs(parser)
        args = parser.parse_args(argv)
        pw.config.fromArgs(args)
        return pw
    
    def getMwApp(self,argv=None,forceRebuild:bool=False):
        """
        get the mediaWikiApp
        """
        if not argv:
            argv=self.argv
        self.pw=self.getProfiWiki(argv)
        self.pw.config.forceRebuild=forceRebuild
        mwApp=self.pw.getMwApp(withGenerate=forceRebuild)
        return mwApp
    
    def doStartMwApp(self,mwApp,forceRebuild:bool=True):
        """
        start MW App
        """
        if forceRebuild:
            mwApp.down(forceRebuild=forceRebuild)
        mwApp.start(forceRebuild=forceRebuild,withInitDB=forceRebuild)
        mwApp.check()
    
    def startMwApp(self):
        """
        start mediawiki application
        """
        forceRebuild=True
        mwApp=self.getMwApp(forceRebuild=forceRebuild)
        self.doStartMwApp(mwApp)
        return mwApp
    
    def test_system(self):
        """
        test system pre requisites
        """
        info=self.pw.system_info()
        debug=True
        if debug:
            print(info)
            
    def test_create(self):
        """
        test creating a wiki
        """
        # remember the docker application
        self.mwApp=self.startMwApp()
        
    def test_logo(self):
        """
        test the logo
        """
        return 
        argv=["--container_name","logotest","--basePort","11001","--sqlBasePort","11002"]
        mwApp=self.getMwApp(argv=argv,forceRebuild=True)
        ls_path=f"{mwApp.dockerPath}/Localsettings.php"
        print (f"checking {ls_path} to exist for {mwApp.config.as_dict()}")
        self.assertTrue(os.path.isfile(ls_path))
        with open(ls_path) as ls_file:
            ls_text= ls_file.read()
            self.assertTrue("Profiwikiicon.png" in ls_text)
        # looking on page itself is version dependent and
        # image is hidden in css /background
        #url=mwApp.config.url
        #main_page_url=f"{url}/index.php/Main_Page"
        #web_scrape=WebScrape()
        #soup=web_scrape.getSoup(main_page_url,showHtml=debug)
        #    self.mwApp=self.startMwApp()
        pass
        
    def test_install_plantuml(self):
        """
        test installing plantuml
        
        takes ~108 secs on laptop
        """
        if self.mwApp is None:
            self.mwApp=self.startMwApp()
        pmw,_pdb=self.pw.getProfiWikiContainers(self.mwApp)
        pmw.install_plantuml()
        pass
    
    def test_install_fontawesome(self):
        """
        test installing font awesome
        
        """
        if self.mwApp is None:
            self.mwApp=self.startMwApp()
        pmw,_pdb=self.pw.getProfiWikiContainers(self.mwApp)
        pmw.install_fontawesome()
    
    def test_killremove(self):
        """
        test kill and remove a container
        """
        if self.mwApp is None:
            self.mwApp=self.startMwApp()
        pmw,pdb=self.pw.getProfiWikiContainers(self.mwApp)
        pmw.killremove()
        pdb.killremove()
        